/**
 * 2009, Digitalis Informatica. All rights reserved. Distribuicao e Gestao de Informatica, Lda. Estrada de Paco de Arcos
 * num.9 - Piso -1 2780-666 Paco de Arcos Telefone: (351) 21 4408990 Fax: (351) 21 4408999 http://www.digitalis.pt
 */

package pt.digitalis.utils.common;

import java.util.ArrayList;
import java.util.LinkedHashMap;
import java.util.List;
import java.util.Map;
import java.util.Map.Entry;
import java.util.Set;

/**
 * Collection objects utility methods
 * 
 * @author Pedro Viegas <a href="mailto:pviegas@digitalis.pt">pviegas@digitalis.pt</a><br/>
 * @created Jan 28, 2009
 */
public class CollectionUtils {

    /**
     * Convert a key-value list from string representation to map representation
     * 
     * @param mapValue
     *            the value to convert
     * @return the converted map in string representation
     */
    static public String keyValueMapToString(Map<String, String> mapValue)
    {
        return keyValueMapToString(mapValue, ",", "=");
    }

    /**
     * Convert a key-value list from string representation to map representation
     * 
     * @param mapValue
     *            the value to convert
     * @param separatorChar
     *            the separator character
     * @param assignmentChar
     *            the assignment separator character
     * @return the converted map in string representation
     */
    static public String keyValueMapToString(Map<String, String> mapValue, String separatorChar, String assignmentChar)
    {
        if (mapValue == null)
            return null;

        StringBuffer buffer = new StringBuffer();
        boolean first = true;

        for (Entry<String, String> entry: mapValue.entrySet())
        {
            if (!first)
                buffer.append(separatorChar);

            buffer.append(entry.getKey());
            buffer.append(assignmentChar);
            buffer.append(entry.getValue());

            first = false;
        }

        return buffer.toString();
    }

    /**
     * Convert a key-value list from map representation to string representation
     * 
     * @param stringValue
     *            the value to convert
     * @return the converted map in string representation
     */
    static public Map<String, String> keyValueStringToMap(String stringValue)
    {
        return keyValueStringToMap(stringValue, ",", "=");
    }

    /**
     * Convert a key-value list from map representation to string representation
     * 
     * @param stringValue
     *            the value to convert
     * @param separatorChar
     *            the separator character
     * @param assignmentChar
     *            the assignment separator character
     * @return the converted map in string representation
     */
    static public Map<String, String> keyValueStringToMap(String stringValue, String separatorChar,
            String assignmentChar)
    {
        if (stringValue == null)
            return null;

        String[] valuePairs = stringValue.split("\\" + separatorChar);
        Map<String, String> result = new LinkedHashMap<String, String>();

        for (String pair: valuePairs)
        {
            String[] values = pair.split("\\" + assignmentChar);

            if (values.length == 2)
                result.put(values[0], values[1]);
        }

        return result;
    }

    /**
     * Convert a key-value list from map representation to string representation
     * 
     * @param stringValue
     *            the value to convert
     * @return the converted map in string representation
     */
    static public Map<String, Object> keyValueStringToMapObject(String stringValue)
    {
        return keyValueStringToMapObject(stringValue, ",", "=");
    }

    /**
     * Convert a key-value list from map representation to string representation
     * 
     * @param stringValue
     *            the value to convert
     * @param separatorChar
     *            the separator character
     * @param assignmentChar
     *            the assignment separator character
     * @return the converted map in string representation
     */
    static public Map<String, Object> keyValueStringToMapObject(String stringValue, String separatorChar,
            String assignmentChar)
    {
        if (stringValue == null)
            return null;

        String[] valuePairs = stringValue.split("\\" + separatorChar);
        Map<String, Object> result = new LinkedHashMap<String, Object>();

        for (String pair: valuePairs)
        {
            String[] values = pair.split("\\" + assignmentChar);

            if (values.length == 2)
                result.put(values[0], values[1]);
        }

        return result;
    }

    /**
     * Converts any given list of objects to a comma separated string os their string representation
     * 
     * @param list
     *            the list to convert
     * @return the converted string
     */
    static public String listToCommaSeparatedString(List<? extends Object> list)
    {
        StringBuffer buffer = null;

        for (Object value: list)
        {

            if (value != null)
            {
                if (buffer == null)
                    buffer = new StringBuffer();
                else
                    buffer.append(",");

                buffer.append(value.toString());
            }
        }

        if (buffer == null)
            return "";
        else
            return buffer.toString();
    }

    /**
     * Converts any given list of objects to a comma separated string os their string representation
     * 
     * @param list
     *            the list to convert
     * @param beanPropertyName
     *            the name of the property to use for the return list
     * @return the converted string
     */
    static public String listToCommaSeparatedString(List<? extends Object> list, String beanPropertyName)
    {
        StringBuffer buffer = null;

        for (Object value: list)
        {
            if (value != null)
            {
                Object propertyValue = BeanInspector.getValue(value, beanPropertyName);

                if (propertyValue != null)
                {
                    if (buffer == null)
                        buffer = new StringBuffer();
                    else
                        buffer.append(",");

                    buffer.append(value.toString());
                }
            }
        }

        if (buffer == null)
            return "";
        else
            return buffer.toString();
    }

    /**
     * Converts any given set of objects to a comma separated string os their string representation
     * 
     * @param itemSet
     *            the set to convert
     * @return the converted string
     */
    static public String setToCommaSeparatedString(Set<? extends Object> itemSet)
    {
        return listToCommaSeparatedString(new ArrayList<Object>(itemSet));
    }
}
