/**
 * 2008, Digitalis Informatica. All rights reserved. Distribuicao e Gestao de Informatica, Lda. Estrada de Paco de Arcos
 * num.9 - Piso -1 2780-666 Paco de Arcos Telefone: (351) 21 4408990 Fax: (351) 21 4408999 http://www.digitalis.pt
 */

package pt.digitalis.utils.common;

import java.util.ArrayList;
import java.util.List;

/**
 * Numeric types utility methods
 * 
 * @author Pedro Viegas <a href="mailto:pviegas@digitalis.pt">pviegas@digitalis.pt</a><br/>
 * @created 2008/12/16
 */
public class NumericUtils {

    /** Java number types */
    final static private List<String> numberClasses = new ArrayList<String>() {

        /**  */
        private static final long serialVersionUID = -2696002481046548758L;

        {
            add(byte.class.getSimpleName());
            add(int.class.getSimpleName());
            add(long.class.getSimpleName());
            add(float.class.getSimpleName());
            add(double.class.getSimpleName());
            add(Number.class.getSimpleName());
            add(Byte.class.getSimpleName());
            add(Short.class.getSimpleName());
            add(Integer.class.getSimpleName());
            add(Long.class.getSimpleName());
            add(Float.class.getSimpleName());
            add(Double.class.getSimpleName());
        }
    };

    /**
     * Verifies if a given class is a numeric type
     * 
     * @param clazz
     *            the class to verify
     * @return T if it is a number type
     */
    static public boolean isNumber(Class<?> clazz)
    {
        return (numberClasses.contains(clazz.getSimpleName()));
    }

    /**
     * Verifies if a given object is a number
     * 
     * @param obj
     *            the object to verify
     * @return T if it is a number
     */
    static public boolean isNumber(Object obj)
    {
        if (obj == null)
            return false;
        else
            return isNumber(obj.getClass());
    }

    /**
     * Convert any number object to a Double
     * 
     * @param number
     *            the number object to convert
     * @return the Double object
     */
    static public Double toDouble(Number number)
    {
        if (number == null)
            return null;

        else
            return number.doubleValue();
    }

    /**
     * Convert any number object to a Integer
     * 
     * @param number
     *            the number object to convert
     * @return the Integer object
     */
    static public Integer toInteger(Number number)
    {
        if (number == null)
            return null;

        else
            return number.intValue();
    }

    /**
     * Convert any number object to a Long
     * 
     * @param number
     *            the number object to convert
     * @return the Long object
     */
    static public Long toLong(Number number)
    {
        if (number == null)
            return null;

        else
            return number.longValue();
    }

    /**
     * Convert any object to a Long
     * 
     * @param object
     *            the string object to convert
     * @return the Long object
     */
    static public Long toLong(Object object)
    {
        if (object == null)
            return null;

        else
            try
            {
                return Long.parseLong(object.toString());
            }
            catch (Exception e)
            {
                System.out.println("This Object/String is not a Number");
                return null;
            }
    }
}
