/**
 * 2007, Digitalis Informatica. All rights reserved. Distribuicao e Gestao de Informatica, Lda. Estrada de Paco de Arcos
 * num.9 - Piso -1 2780-666 Paco de Arcos Telefone: (351) 21 4408990 Fax: (351) 21 4408999 http://www.digitalis.pt
 */
package pt.digitalis.utils.common;

import java.util.Map;

/**
 * @author Pedro Viegas <a href="mailto:pviegas@digitalis.pt">pviegas@digitalis.pt</a>
 */
public class StringUtils {

    /**
     * Converts a string in CamalCase to a normal sentence string. Ex: "SampleStringInCamelCase" to "Sample String In
     * Camel Case"
     * 
     * @param source
     *            the source string in camel case
     * @return the converted string
     */
    static public String camelCaseToString(String source)
    {
        if (source == null)
            return null;

        StringBuffer buffer = new StringBuffer();
        boolean nextIsUpper = false;

        for (char character: source.toCharArray())
        {
            if (Character.isUpperCase(character) && (buffer.length() > 0))
                buffer.append(' ');

            if ((buffer.length() == 0) || nextIsUpper)
            {
                buffer.append(Character.toUpperCase(character));
                nextIsUpper = false;

            }
            else
            {
                if (".-_".contains(Character.toString(character)))
                {
                    nextIsUpper = true;
                    buffer.append(' ');
                }
                else
                {
                    buffer.append(character);
                }
            }
        }

        return buffer.toString();
    }

    /**
     * Fills the given string with character on the right
     * 
     * @param source
     *            the source string
     * @param size
     *            the size to fill to
     * @param fillString
     *            the string used for filling
     * @return the filled string
     */
    static public String fillString(String source, int size, String fillString)
    {
        return fillStringRight(source, size, fillString);
    }

    /**
     * Fills the given string with character on the right
     * 
     * @param source
     *            the source string
     * @param size
     *            the size to fill to
     * @param fillString
     *            the string used for filling
     * @return the filled string
     */
    static public String fillStringLeft(String source, int size, String fillString)
    {

        if (source.length() < size)
            return getRepeatedString(fillString, size - source.length()) + source;
        else
            return source.substring(0, size);
    }

    /**
     * Fills the given string with character on the right
     * 
     * @param source
     *            the source string
     * @param size
     *            the size to fill to
     * @param fillString
     *            the string used for filling
     * @return the filled string
     */
    static public String fillStringRight(String source, int size, String fillString)
    {

        if (source.length() < size)
            return source + getRepeatedString(fillString, size - source.length());
        else
            return source.substring(0, size);
    }

    /**
     * Retrieves the first line of a multi-line string
     * 
     * @param str
     *            the multi-line string
     * @return the first line of str
     */
    static public String getFirstLine(String str)
    {
        if (hasSeveralLines(str))
            return str.substring(0, str.indexOf("\n"));
        else
            return str;
    }

    /**
     * Retrieves all lines starting on a given one of a multi-line string
     * 
     * @param str
     *            the multi-line string
     * @param startLineNumber
     *            the first line to return
     * @return a string with all lines of str starting on startLineNumber
     */
    static public String getLinesBeginingOn(String str, int startLineNumber)
    {
        int startPos = 0;

        while (startLineNumber > 1 && startPos != -1)
        {
            startPos = str.indexOf("\n", startPos);

            if (startPos != -1)
                startPos++;

            startLineNumber--;
        }

        if (startPos == -1)
            return "";
        else
            return str.substring(startPos);
    }

    /**
     * Returns a String with the given size calculated by repeating the necessary number of times the given string
     * pattern. The string is truncated to the given size.
     * 
     * @param source
     *            the pattern string to repeat
     * @param size
     *            the desired number of characters
     * @return the calculated string
     */
    static public String getRepeatedString(String source, int size)
    {
        String buffer = "";

        while (buffer.length() < size)
            buffer += source;

        return buffer.substring(0, size);
    }

    /**
     * Determines if a string has more than one line of text
     * 
     * @param str
     *            the string to parse
     * @return T if more than one line is present
     */
    static public boolean hasSeveralLines(String str)
    {
        if (str == null)
            return false;
        else
            return (str.indexOf("\n") != -1);
    }

    /**
     * Return the leftmost indicated number of characters from the string.
     * 
     * @param str
     *            the original string
     * @param size
     *            the number of characters to retrieve
     * @return the leftmost characters
     */
    static public String left(String str, int size)
    {
        if (str == null || str.length() <= size)
            return str;
        else
            return str.substring(0, size);
    }

    /**
     * Validates the given string for value assigned. Will return it if it is not null, or the null alternative
     * otherwise. Avoids the common {@link NullPointerException}
     * 
     * @param obj
     *            the object validate
     * @param nullValue
     *            the value to return if obj is null
     * @return the converted object to string or null
     */
    public static String nvl(String obj, String nullValue)
    {
        return (obj == null) ? nullValue : obj;
    }

    /**
     * Parses a message and substitutes all given keys for their values
     * 
     * @param message
     *            the message template
     * @param substitutions
     *            the map of key/value to replace
     * @return the parsed message with the values
     */
    static public String parseMessage(String message, Map<String, String> substitutions)
    {

        String result = message;

        for (String key: substitutions.keySet())
        {
            String value = substitutions.get(key);
            if (value == null)
                value = "null";

            result = result.replace("${" + key + "}", value);
        }
        return result;
    }

    /**
     * Removes the accents from a string.
     * 
     * @param _strToConvert
     *            string to strip off of accents
     * @return the string without accents
     */
    public final static String removeAccents(String _strToConvert)
    {

        String res = _strToConvert;
        final int[] ACCENTS = {193, 194, 195, 201, 202, 205, 211, 213, 212, 218, 199, 225, 226, 227, 233, 234, 237,
                            243, 245, 244, 250, 252, 231};
        final String CHARS = "AAAEEIOOOUCaaaeeiooouuc";
        for (int i = 0; i < ACCENTS.length; i++)
        {
            res = res.replaceAll(((char) ACCENTS[i]) + "", CHARS.charAt(i) + "");
        }
        return res;

        // public static void main(String args[]) {
        //
        // String s = ",,,,,,,,,,,,,,,,,,,,,";
        //
        //
        //
        // s = s.replaceAll("[]","e");
        //
        // s = s.replaceAll("[]","u");
        //
        // s = s.replaceAll("[]","i");
        //
        // s = s.replaceAll("[]","a");
        //
        // s = s.replaceAll("","o");
        //
        //
        //
        // s = s.replaceAll("[]","E");
        //
        // s = s.replaceAll("[]","U");
        //
        // s = s.replaceAll("[]","I");
        //
        // s = s.replaceAll("[]","A");
        //
        // s = s.replaceAll("","O");
        //
        //
        //
        // System.out.println(s);

        // output : E,E,E,E,U,U,I,I,A,A,O,e,e,e,e,u,u,i,i,a,a,o

    }

    /**
     * Return the rightmost indicated number of characters from the string.
     * 
     * @param str
     *            the original string
     * @param size
     *            the number of characters to retrieve
     * @return the rightmost characters
     */
    static public String right(String str, int size)
    {
        if (str == null || str.length() <= size)
            return str;
        else
            return str.substring(str.length() - size);
    }

    /**
     * Converts the first character of a string to lower case
     * 
     * @param source
     *            the source string
     * @return the converted string
     */
    static public String toLowerFirstChar(String source)
    {
        StringBuffer buffer = new StringBuffer();

        if (source.length() > 1)
        {
            buffer.append(left(source, 1).toLowerCase());
            buffer.append(source.substring(1));

            return buffer.toString();
        }
        else
            return source.toLowerCase();
    }

    /**
     * Converts an object to string representation, or null. Avoids the common {@link NullPointerException}
     * 
     * @param obj
     *            the object to convert
     * @return the converted object to string or null
     */
    public static String toStringOrNull(Object obj)
    {
        if (obj == null)
            return null;
        else
            return obj.toString();
    }

    /**
     * Converts the first character of a string to upper case
     * 
     * @param source
     *            the source string
     * @return the converted string
     */
    static public String toUpperFirstChar(String source)
    {
        StringBuffer buffer = new StringBuffer();

        if (source.length() > 1)
        {
            buffer.append(left(source, 1).toUpperCase());
            buffer.append(source.substring(1));

            return buffer.toString();
        }
        else
            return source.toUpperCase();
    }
}
