/**
 * 2009, Digitalis Informatica. All rights reserved. Distribuicao e Gestao de Informatica, Lda. Estrada de Paco de Arcos
 * num.9 - Piso -1 2780-666 Paco de Arcos Telefone: (351) 21 4408990 Fax: (351) 21 4408999 http://www.digitalis.pt
 */

package pt.digitalis.utils.common;

import java.io.BufferedReader;
import java.io.ByteArrayInputStream;
import java.io.ByteArrayOutputStream;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.io.FileReader;
import java.io.IOException;
import java.io.InputStream;
import java.util.zip.ZipEntry;
import java.util.zip.ZipOutputStream;

import org.apache.commons.io.IOUtils;

/**
 * @author Pedro Viegas <a href="mailto:pviegas@digitalis.pt">pviegas@digitalis.pt</a><br/>
 * @created 2010/09/20
 */
public class ZipPackagerUtil {

    /**  */
    private String outputZipFileName;

    /** OutputStream do ficheiro zip a criar em disco */
    private ZipOutputStream zipOut;

    /**
     * @param outputZipFileName
     */
    public ZipPackagerUtil(String outputZipFileName)
    {
        this.outputZipFileName = outputZipFileName;
    }

    /**
     * Method to add an entry in zip file from file in disk
     * 
     * @param sourceBuffer
     * @param entryName
     * @return true if entry is sucessfull created
     * @exception IOException
     */
    public boolean addEntryFromBuffer(byte[] sourceBuffer, String entryName) throws IOException
    {
        ZipOutputStream out = getZipOutputStream();
        byte[] buffer = new byte[1024];
        int len;
        ByteArrayInputStream input = null;

        try
        {

            input = new ByteArrayInputStream(sourceBuffer);

            out.putNextEntry(new ZipEntry(entryName));
            while ((len = input.read(buffer)) > 0)
            {
                out.write(buffer, 0, len);
            }
            out.closeEntry();
            out.flush();

        }
        finally
        {
            if (input != null)
            {
                try
                {
                    input.close();
                }
                catch (Exception e)
                {
                    e.printStackTrace();
                }
            }
        }

        return true;
    }

    /**
     * Method to add an entry in zip file from file in disk
     * 
     * @param sourceBuffer
     * @param entryName
     * @return true if entry is sucessfull created
     * @exception IOException
     */
    public boolean addEntryFromBuffer(StringBuffer sourceBuffer, String entryName) throws IOException
    {
        return addEntryFromBuffer(sourceBuffer.toString().getBytes(), entryName);
    }

    /**
     * Method to add an entry in zip file from file in disk
     * 
     * @param stream
     * @param entryName
     * @return true if entry is successful created
     * @exception IOException
     */
    public boolean addEntryFromStream(InputStream stream, String entryName) throws IOException
    {
        ZipOutputStream out = getZipOutputStream();
        ByteArrayOutputStream doc = null;

        try
        {
            out.putNextEntry(new ZipEntry(entryName));
            out.write(IOUtils.toByteArray(stream));
            out.closeEntry();
            out.flush();
        }
        finally
        {
            if (doc != null)
            {
                try
                {
                    doc.close();
                }
                catch (Exception e)
                {
                    e.printStackTrace();
                }
            }
        }

        return true;
    }

    /**
     * Method to add an entry in zip file from file in disk
     * 
     * @param sourceFileName
     * @param entryName
     * @return true if entry is sucessfull created
     * @exception IOException
     */
    public boolean addFileToZip(String sourceFileName, String entryName) throws IOException
    {

        ZipOutputStream out = getZipOutputStream();

        BufferedReader localBufferedReader = null;
        String fileLine = null;

        try
        {
            out.putNextEntry(new ZipEntry(entryName));

            localBufferedReader = new BufferedReader(new FileReader(sourceFileName));

            fileLine = localBufferedReader.readLine();
            while (fileLine != null)
            {
                out.write((fileLine + "\n").getBytes());
                out.flush();
                fileLine = localBufferedReader.readLine();
            }
            out.closeEntry();

        }
        finally
        {
            if (localBufferedReader != null)
            {
                try
                {
                    localBufferedReader.close();
                }
                catch (Exception e)
                {
                    e.printStackTrace();
                }
            }
        }

        return true;
    }

    /**
     * Method to close the zip file
     * 
     * @exception IOException
     *                when an error occurs; description added in several situations
     */
    public void closeZipFile() throws IOException
    {
        if (zipOut != null)
            zipOut.close();
    }

    /**
     * Method to get the zip file in the response
     * 
     * @return byte[] to be set in the response
     * @exception IOException
     *                when an error occurs; description added in several situations
     */
    public byte[] getZipFileContent() throws IOException
    {
        closeZipFile();

        FileInputStream is = null;
        byte[] bytes = null;

        try
        {
            File file = new File(outputZipFileName);

            if (!file.exists())
                throw new IOException("File " + outputZipFileName + " does not exist!");

            is = new FileInputStream(file);
            long length = file.length();
            if (length > Integer.MAX_VALUE)
            {
                // File is too large
                throw new IOException("File is too large!");
            }
            bytes = new byte[(int) length];
            // Read in the bytes
            int offset = 0;
            int numRead = 0;

            while (offset < bytes.length && (numRead = is.read(bytes, offset, bytes.length - offset)) >= 0)
            {
                offset += numRead;
            }

            // Ensure all the bytes have been read in
            if (offset < bytes.length)
            {
                throw new IOException("Could not completely read file " + file.getName());
            }

        }
        finally
        {
            if (is != null)
            {
                try
                {
                    is.close();
                }
                catch (Exception e)
                {
                    e.printStackTrace();
                }
            }
        }
        return bytes;

    }

    /**
     * Method to get the zip file strem
     * 
     * @return byte[] to be set in the response
     * @exception IOException
     *                when an error occurs; description added in several situations
     */
    public InputStream getZipFileStream() throws IOException
    {
        closeZipFile();

        File file = new File(outputZipFileName);

        if (!file.exists())
            throw new IOException("File " + outputZipFileName + " does not exist!");

        return new FileInputStream(file);

    }

    /**
     * @return ZipOutputStream
     * @throws IOException
     */
    protected ZipOutputStream getZipOutputStream() throws IOException
    {
        if (zipOut == null)
        {
            File outFile = new File(outputZipFileName);
            if (outFile.exists())
                outFile.delete();

            zipOut = new ZipOutputStream(new FileOutputStream(outputZipFileName));
        }

        return zipOut;
    }

}
