/**
 * 2007, Digitalis Informatica. All rights reserved. Distribuicao e Gestao de Informatica, Lda. Estrada de Paco de Arcos
 * num.9 - Piso -1 2780-666 Paco de Arcos Telefone: (351) 21 4408990 Fax: (351) 21 4408999 http://www.digitalis.pt
 */
package pt.digitalis.utils.common.collections;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.Map.Entry;

/**
 * /** This class implements a
 * 
 * @param <Key>
 *            the type of the key
 * @param <Value>
 *            the type of the value
 * @author Pedro Viegas <a href="mailto:pviegas@digitalis.pt">pviegas@digitalis.pt</a><br/>
 * @created Feb 18, 2009
 */
public class IndexedHashMap<Key, Value> extends HashMap<Key, Value> {

    /**  */
    private static final long serialVersionUID = 8889037204459685855L;

    /** The index for the map */
    List<Key> index = new ArrayList<Key>();

    /**
     * @see java.util.HashMap#clear()
     */
    @Override
    synchronized public void clear()
    {
        super.clear();
        index.clear();
    }

    /**
     * Retrieves the element in the given position
     * 
     * @param indexPos
     *            the position of the element
     * @return the value at the given position
     */
    synchronized public Value getByPosition(int indexPos)
    {
        return get(index.get(indexPos));
    }

    /**
     * Retrieves the keys in an ordered list
     * 
     * @return the key list
     */
    synchronized public List<Key> getOrderedKeys()
    {

        return index;
    }

    /**
     * Retrieves the values in an ordered list
     * 
     * @return the values list
     */
    synchronized public List<Value> getOrderedValues()
    {

        List<Value> values = new ArrayList<Value>();

        for (Key key: index)
            values.add(get(key));

        return values;
    }

    /**
     * @see java.util.HashMap#put(java.lang.Object, java.lang.Object)
     */
    @Override
    synchronized public Value put(Key key, Value value)
    {
        if (!index.contains(key))
            index.add(key);

        return super.put(key, value);
    }

    /**
     * Adds a set of entries to the current map. Preserving the original add order
     * 
     * @param map
     *            the map to add
     */
    synchronized public void putAll(IndexedHashMap<Key, Value> map)
    {
        for (Key keyID: map.getOrderedKeys())
            put(keyID, map.get(keyID));
    }

    /**
     * @see java.util.HashMap#putAll(java.util.Map)
     */
    @Override
    synchronized public void putAll(Map<? extends Key, ? extends Value> m)
    {
        for (Entry<? extends Key, ? extends Value> entry: m.entrySet())
            put(entry.getKey(), entry.getValue());
    }

    /**
     * @see java.util.HashMap#remove(java.lang.Object)
     */
    @Override
    synchronized public Value remove(Object key)
    {
        if (index.remove(key))
            return super.remove(key);
        else
            return null;
    }
}
